<?php

namespace Netzmacher\Browser\Utility\FrontendEditing;

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2017 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for prefilling powermail fields
 *
 * @package TYPO3
 * @subpackage browser
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @internal #i0293
 * @version 8.0.21
 * @since 8.0.17
 */
class Prefill
{

	/**
	 * @var array Array[ table ] = uid
	 */
	private $_aIds;

	/**
	 * @var array Current Powermail marker
	 */
	private $_aPowermailMarker;

	/**
	 * @var array TypoScript setup
	 */
	private $_aSettings;

	/**
	 * @var TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer
	 */
	private $_oCObj;

	/**
	 * @var Netzmacher\Browser\Utility\FrontendEditing\DRS
	 */
	private $_oDRS;

	/**
	 * @var Netzmacher\Browser\Utility\FrontendEditing\TCA
	 */
	private $_oTCA;

	/**
	 * @var TYPO3\CMS\Extbase\Service\TypoScriptService
	 */
	private $_oTypoScriptService;

	/**
	 * @var string
	 */
	private $_sAction;

	/**
	 * _DRSprompt( ) :
	 *
	 * @param string $prompt
	 * @param int $line
	 * @param int $type
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 *
	 */
	private function _DRSprompt( $prompt, $line = __LINE__, $type = 0 )
	{
		if( !$this->_oDRS->getDrsFrontendEditing() )
		{
			return;
		}
		GeneralUtility::devlog( '[INFO/FE] ' . $prompt, __CLASS__ . '#' . $line, $type );
	}

	/**
	 * _DRSpromptData( ) :
	 *
	 * @param array $data
	 * @param int $line
	 * @param int $type
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 *
	 */
	private function _DRSpromptData( $data, $line = __LINE__, $type = 0 )
	{
		if( !$this->_oDRS->getDrsFrontendEditing() )
		{
			return;
		}

		foreach( ( array ) $data AS $field => $value )
		{
			$aPrompt[] = $field . ': "' . $value . '"';
		}
		$prompt = implode( ', ', ( array ) $aPrompt );
		//$prompt = 'Fields are added to cObj. Fields are available by TypoScript! ' . $prompt;
		$this->_DRSprompt( $prompt, $line, $type );
	}

	/**
	 * _SQLError( ) :
	 *
	 * @param string $query
	 * @param int $line
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _SQLError( $query, $line = __LINE__ )
	{
		$sqlError = $GLOBALS[ 'TYPO3_DB' ]->sql_error();
		if( empty( $sqlError ) )
		{
			return;
		}

		$sqlErrno = $GLOBALS[ 'TYPO3_DB' ]->sql_errno();
		$header = 'SQL-ERROR';
		$text = ''
						. '<p>'
						. '  Error: ' . $sqlError . '<br />'
						. '  Error number: ' . $sqlErrno . '<br />'
						. '  Query: ' . $query . '<br />'
						. '</p>'
		;
		$this->_zzDieWiPrompt( $header, $text, __METHOD__, $line );
	}

	/**
	 * _SQLSelect( ) :
	 *
	 * @param string $table
	 * @param integer $uid
	 * @param string $where
	 * @param int $line
	 * @param boolean $bEnableFields
	 * @param boolean $bPromptQueryOnly
	 * @return array	rows
	 * @access private
	 * @version 8.0.18
	 * @since 7.4.0
	 */
	private function _SQLSelect( $table, $uid = NULL, $where = 1, $line = __LINE__, $bEnableFields = false, $bPromptQueryOnly = false )
	{
		$query = $GLOBALS[ 'TYPO3_DB' ]->SELECTquery(
						'*'
						, $table
						, $this->_SQLSelectWhere( $table, $uid, $where, $bEnableFields )
		);
		//var_dump( __METHOD__, __LINE__, $query );

		switch( $bPromptQueryOnly )
		{
			case(TRUE):
				$this->_DRSprompt( 'Query: ' . $query, $line );
				var_dump( __METHOD__, __LINE__, $query );
				break;
			case(FALSE):
			default:
				$this->_DRSprompt( 'Query: ' . $query, $line );
				$res = $GLOBALS[ 'TYPO3_DB' ]->exec_SELECTquery(
								'*'
								, $table
								, $this->_SQLSelectWhere( $table, $uid, $where, $bEnableFields )
				);
				$this->_SQLError( $query, $line );
				while( $row = $GLOBALS[ 'TYPO3_DB' ]->sql_fetch_assoc( $res ) )
				{
					$rows[] = $row;
				}

				//var_dump( __METHOD__, __LINE__, $rows );
				return $rows;
		}
	}

	/**
	 * _SQLSelectWhere( ) :
	 *
	 * @param string $table
	 * @param integer $uid
	 * @param string $where
	 * @param boolean $bEnableFields
	 * @return array	rows
	 * @access private
	 * @version 8.0.18
	 * @since 7.4.0
	 */
	private function _SQLSelectWhere( $table, $uid = null, $where = null, $bEnableFields = false )
	{
		if( empty( $uid ) )
		{
			$sUid = 1;
		}
		else
		{
			$sUid = 'uid = ' . $uid;
		}

		if( !empty( $where ) )
		{
			$where = ' AND ' . $where;
		}

		$pid = $this->_getPid( $table );
		if( !empty( $pid ) )
		{
			$pid = 'AND pid = ' . $pid;
		}

		$where = $sUid
						. $where
						. $this->_SQLSelectWhereEnableFields( $table, $bEnableFields ) . ' '
						. $pid
		;

		return $where;
	}

	/**
	 * _SQLSelectWhereEnableFields( ) :
	 *
	 * @param string $table
	 * @param boolean $bEnableFields
	 * @return array	rows
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _SQLSelectWhereEnableFields( $table, $bEnableFields )
	{
		if( !$bEnableFields )
		{
			return;
		}

		return $this->_oCObj->enableFields( $table );
	}

	/**
	 * _feuserIsOwner :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _feuserIsOwner( $row )
	{
		if( $row[ 'fe_cruser_id' ] == $GLOBALS[ 'TSFE' ]->fe_user->user[ 'uid' ] )
		{
			$prompt = 'Current fe_user is owner of the current record. Workflow will followed.';
			$this->_DRSprompt( $prompt, __LINE__, -1 );
			return true;
		}
		$prompt = 'Current fe_user isn\'t owner of the current record. Workflow won\'t followed.';
		$this->_DRSprompt( $prompt, __LINE__, 2 );
		return false;
	}

	/**
	 * _getPid( ) :
	 *
	 * @param string $table
	 * @param integer $uid
	 * @param int $line
	 * @param boolean $bPromptQueryOnly
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _getPid( $table )
	{
		if( !isset( $this->_aSettings[ 'pid' ][ $table ] ) )
		{
			return;
		}
		$typoscriptPlain = $this->_aSettings[ 'pid' ][ $table ];
		$name = $typoscriptPlain[ '_typoScriptNodeValue' ];
		$conf = $this->_oTypoScriptService->convertPlainArrayToTypoScriptArray( $typoscriptPlain );

		$pid = ( int ) $this->_oCObj->cObjGetSingle( $name, $conf );
		return $pid;
	}

	/**
	 * _init( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _init()
	{
		$this->_initObjectCObj();
		$this->_initObjectTypoScriptService();
	}

	/**
	 * _initObjectCObj( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _initObjectCObj()
	{
		$this->_oObjectManager = GeneralUtility::makeInstance( 'TYPO3\\CMS\\Extbase\\Object\\ObjectManager' );
		$this->_oCObj = $this->_oObjectManager->get( 'TYPO3\\CMS\\Frontend\\ContentObject\\ContentObjectRenderer' );
	}

	/**
	 * _initObjectTypoScriptService( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _initObjectTypoScriptService()
	{
		$this->_oTypoScriptService = GeneralUtility::makeInstance( 'TYPO3\\CMS\\Extbase\\Service\\TypoScriptService' );
	}

	/**
	 * _isForeignTable( ) :
	 *
	 * @param array	$row
	 * @return array	$row
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _isForeignTable( $table, $uid, $row )
	{
		//var_dump( __METHOD__, __LINE__, $table, $uid, $this->_aPowermailMarker );

		foreach( $this->_aPowermailMarker as $tableField )
		{
			list($pmTable, $field) = explode( '__', $tableField );
			if( $pmTable != $table )
			{
				continue;
			}
			if( !in_array( $field, array_keys( ( array ) $row ) ) )
			{
				continue;
			}
			$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
			if( !isset( $config[ 'foreign_table' ] ) )
			{
				continue;
			}
			$this->_processForeignTable( $uid, $table, $field );
		}
	}

	/**
	 * _processForeignTable( ) :
	 *
	 * @param string $row
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _processForeignTable( $uid, $table, $field )
	{
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		switch( TRUE )
		{
			case(isset( $config[ 'MM' ] )):
				$this->_processForeignTableMM( $uid, $table, $field );
				break;
			case($config[ 'foreign_table' ] == 'sys_file_reference'):
				$this->_processFAL( $table, $field );
				break;
			default:
				$this->_processForeignTableCSV( $table, $field );
				break;
		}
		return;
	}

	/**
	 * _processForeignTableMM( ) :
	 *
	 * @param string $row
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _processForeignTableMM( $uid, $table, $field )
	{
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		//var_dump( __METHOD__, __LINE__, $config, $config[ 'MM' ], $config[ 'MM_match_fields' ], $table, $field );

		$andWhere = '';
		foreach( ( array ) $config[ 'MM_match_fields' ] as $key => $value )
		{
			$andWhere = $andWhere . ' AND ' . $key . ' = \'' . $value . '\'';
		}
		$tableMM = $config[ 'MM' ];
		$where = 'uid_local = ' . $uid . $andWhere;
		$fRows = $this->_SQLSelect( $tableMM, null, $where, __LINE__, 0, 0 );
		$_aUidForeign = [ ];
		foreach( ( array ) $fRows as $fRow )
		{
			$_aUidForeign[] = $fRow[ 'uid_foreign' ];
		}
//		if( !empty( $_aUidForeign ) )
//		{
			$this->_setPOST( $table, $field, $_aUidForeign );
//		}
	}

	/**
	 * _processFAL( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return array	$row
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _processFAL( $table, $field )
	{
		$prompt = $table . '.' . $field . ' can\'t handled, because current method isn\'t supported: ' . __METHOD__;
		$this->_DRSprompt( $prompt, __LINE__, 2 );
		return $row;
	}

	/**
	 * _processForeignTableCSV( ) :
	 *
	 * @param string $row
	 * @param string $table
	 * @param string $field
	 * @return array	$row
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _processForeignTableCSV( $row, $table, $field )
	{
		$prompt = $table . '.' . $field . ' can\'t handled, because current method isn\'t supported: ' . __METHOD__;
		$this->_DRSprompt( $prompt, __LINE__, 2 );
		return $row;
	}

	/**
	 * _render( ) : Renders the params for prefilling powermail fields.
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.17
	 */
	private function _render()
	{
		switch( $this->_sAction )
		{
			case('add'):
				$this->_renderActionAdd();
				break;
			case('copy'):
				$this->_renderActionCopy();
				break;
			case('update'):
				$this->_renderActionUpdate();
				break;
			default:
				$this->_renderActionDefault();
				break;
		}
	}

	/**
	 * _renderActionAdd( ) : 
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _renderActionAdd()
	{
		$this->_renderPowermail();
		foreach( ( array ) $this->_aPowermailMarker as $tableField )
		{
			list($table, $field) = explode( '__', $tableField );
			$value = $this->_renderCurrentValue( $table, $field );
			if( empty( $value ) )
			{
				continue;
			}
			$this->_setPOST( $table, $field, $value );
		}
	}

	/**
	 * _renderActionCopy( ) : 
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _renderActionCopy()
	{
		$header = 'Not supported';
		$text = ''
						. '<p>'
						. '  action copy isn\'t supported.<br />'
						. '</p>'
		;
		$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
	}

	/**
	 * _renderActionCopy( ) : 
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _renderActionDefault()
	{
		$header = 'Not supported';
		$text = ''
						. '<p>'
						. '  action is empty. This isn\'t supported.<br />'
						. '</p>'
		;
		$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
	}

	/**
	 * renderActionUpdate( ) : Renders the params for prefilling powermail fields.
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.17
	 */
	private function _renderActionUpdate()
	{
		$this->_renderPowermail();
		//var_dump( __METHOD__, __LINE__, $this->_aIds );
		foreach( $this->_aIds as $table => $uid )
		{
			$row = $this->_renderCurrentRow( $table, $uid );
			foreach( ( array ) $row as $field => $value )
			{
				//var_dump( __METHOD__, __LINE__, $field );
				$value = $this->_renderCurrentValue( $table, $field, $value );
//				if($field == 'tx_org_location' or $field == 'tx_org_caltype'){
//					var_dump( __METHOD__, __LINE__, $table, $field, $value );
//				}
				$this->_setPOST( $table, $field, $value );
			}
			$this->_isForeignTable( $table, $uid, $row );
		}
	}

	/**
	 * _renderCurrentFieldImage( ) : This is a workaround! Image value muts be empty.
	 *																Otherwise the Powermail viewhelper runs in an error.
	 *
	 * @return void
	 * @internal #i0311
	 * @access private
	 * @version 8.0.22
	 * @since 8.0.22
	 */
	private function _renderCurrentFieldImage( $table, $field, $value )
	{
		
		return null;
	}

	/**
	 * _renderCurrentRow :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _renderCurrentRow( $table, $uid )
	{
		$rows = $this->_SQLSelect( $table, $uid, null, __LINE__, 0, 0 );
		$this->_renderCurrentRowEmpty( $rows, $table, $uid );
		$row = $rows[ 0 ];
		if( !$this->_feuserIsOwner( $row ) )
		{
			//var_dump(__METHOD__, __LINE__, $table );
			return;
		}
		$this->_renderCurrentRowDRS( $row );
		return $row;
	}

	/**
	 * _renderCurrentRowEmpty :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _renderCurrentRowEmpty( $rows, $table, $uid )
	{
		if( !empty( $rows ) )
		{
			return;
		}
		$prompt = $table . ' with uid ' . $uid . ' hasn\'t any row as result. This isn\'t proper.';
		$this->_DRSprompt( $prompt, __LINE__, 3 );
	}

	/**
	 * _renderCurrentRowDRS :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _renderCurrentRowDRS( $row )
	{
		switch( true )
		{
			case( empty( $row )):
				$prompt = 'Row is empty. Is there any mistake? You can check the SQL query from below.';
				$this->_DRSprompt( $prompt, __LINE__, 2 );
				break;
			case(!empty( $row )):
			default:
				$this->_DRSpromptData( $row, __LINE__, -1 );
				break;
		}
	}

	/**
	 * _renderCurrentValue( ) : 
	 *
	 * @return void
	 * @access private
	 * @version 8.0.22
	 * @since 8.0.21
	 */
	private function _renderCurrentValue( $table, $field, $value = null )
	{
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		switch( true  )
		{
			case( $field == 'image'):
			case( $field == 'images'):
				// This is a workaround!
				$value = $this->_renderCurrentFieldImage( $table, $field, $value );
				break;
			case( $config[ 'type' ] == 'input'):
				$value = $this->_renderCurrentValueInput( $table, $field, $value );
				break;
			default:
				// Do nothing
				break;
		}

		return $value;
	}

	/**
	 * _renderCurrentValueEvalDatetime( ) : 
	 *
	 * @return void
	 * @access private
	 * @version 8.0.22
	 * @since 8.0.22
	 */
	private function _renderCurrentValueEvalDate( $value )
	{
		if( empty( $value ) )
		{
			$value = strtotime( '+1 day +1 hour' );
		}
		$value = strftime( '%x', $value );
		//$value = strftime( '01/29/2017', $value );
		return $value;
	}

	/**
	 * _renderCurrentValueEvalDatetime( ) : 
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _renderCurrentValueEvalDatetime( $value )
	{
		if( empty( $value ) )
		{
			$value = strtotime( '+1 day +1 hour' );
		}
		$value = strftime( '%x %H:00', $value );
		//$value = strftime( '01/29/2017, 06:00 AM', $value );
		return $value;
	}

	/**
	 * _renderCurrentValueInput( ) : 
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _renderCurrentValueInput( $table, $field, $value )
	{
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		switch( true )
		{
			case( strpos( $config[ 'eval' ], 'datetime' ) !== false ):
				$value = $this->_renderCurrentValueEvalDatetime( $value );
				break;
			case( strpos( $config[ 'eval' ], 'date' ) !== false ):
				$value = $this->_renderCurrentValueEvalDate( $value );
				break;
			default:
				// Do nothing
				break;
		}
		return $value;
	}

	/**
	 * _renderPowermail() : Set used Powermail fields / marker
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _renderPowermail()
	{
		$rows = $this->_renderPowermailFields();
		$this->_renderPowermailMarker( $rows );
	}

	/**
	 * _renderPowermailFields : Get used Powermail fields
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _renderPowermailFields()
	{
		$table = 'tx_powermail_domain_model_field';
		$rows = $this->_SQLSelect( $table, null, null, __LINE__ );
		return $rows;
	}

	/**
	 * _renderPowermailMarker : Set used Powermail marker
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _renderPowermailMarker( $rows )
	{
		foreach( ( array ) $rows as $row )
		{
			$this->_aPowermailMarker[] = $row[ 'marker' ];
		}
		//var_dump( __METHOD__, __LINE__, $this->_aPowermailMarker );
		$this->_DRSpromptData( $this->_aPowermailMarker, __LINE__ );
	}

	/**
	 * _requirements( ) :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _requirements()
	{
		if( $this->_sAction == 'add' )
		{
			return true;
		}
		if( !$this->_requirementsLoggedIn() )
		{
			return false;
		}
		if( !$this->_requirementsIds() )
		{
			return false;
		}
		return true;
	}

	/**
	 * _requirementsIds( ) :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _requirementsIds()
	{
		if( !$this->_requirementsParam() )
		{
			return false;
		}

		foreach( array_keys( $this->_aSettings[ 'param' ] ) as $table )
		{
			if( $table == '_typoScriptNodeValue' )
			{
				continue;
			}
			$typoscriptPlain = $this->_aSettings[ 'param' ][ $table ];
			$name = $typoscriptPlain[ '_typoScriptNodeValue' ];
			$conf = $this->_oTypoScriptService->convertPlainArrayToTypoScriptArray( $typoscriptPlain );

			$value = ( int ) $this->_oCObj->cObjGetSingle( $name, $conf );
			if( $value > 0 )
			{
				$this->_aIds[ $table ] = $value;
			}
		}
		if( empty( $this->_aIds ) )
		{
			$prompt = 'requirements aren\'t met: there isn\'t any id (showUid). This seem\'s to be strange.';
			$this->_DRSprompt( $prompt, __LINE__, 2 );
			return false;
		}
		return true;
	}

	/**
	 * _requirementsParam( ) :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _requirementsParam()
	{
		if( isset( $this->_aSettings[ 'param' ] ) )
		{
			return true;
		}

		$header = 'TYPOSCRIPT-ERROR';
		$text = ''
						. '<p>'
						. '  plugin.tx_browser_pi6.settings.param isn\'t set.<br />'
						. '  Please take care for a proper TypoScript!<br />'
						. '</p>'
		;
		$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
		return false;
	}

	/**
	 * _requirementsLoggedIn( ) :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _requirementsLoggedIn()
	{
		if( empty( $GLOBALS[ 'TSFE' ]->fe_user->user ) )
		{
			return false;
		}
		return true;
	}

	/**
	 * _setPOST( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.17
	 */
	private function _setPOST( $table, $field, $value )
	{
		$tableField = $table . '__' . $field;
		if( !in_array( $tableField, ( array ) $this->_aPowermailMarker ) )
		{
			//var_dump(__METHOD__, __LINE__, $tableField);
			return;
		}
		//var_dump(__METHOD__, __LINE__, $tableField, $value);
//		if( empty( $value ) )
//		{
//			return;
//		}
		// 170123, dwildt, 1-/+
		//$_POST[ 'tx_powermail_pi1' ][ $tableField ] = $value;
		$_POST[ 'tx_powermail_pi1' ][ 'field' ][ $tableField ] = $value;
		$prompt = 'Added to POST: ' . $tableField . '[' . $value . ']';
		$this->_DRSprompt( $prompt, __LINE__ );
	}

	/**
	 * _zzDieWiPrompt( ) :
	 *
	 * @access private
	 * @return boolean
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _zzDieWiPrompt( $header, $text, $method, $line )
	{
		$prompt = '
      <h1 style="color:red;">
        ' . $header . '
      </h1>
      ' . $text . '
      <p>
        Sorry for the trouble. Browser - TYPO3 without PHP<br />
        Error occures here: ' . $method . ' at #' . $line . '
      </p>
      <p>
        If you need any help, please visit the
        <a href="http://typo3-browser-forum.de/" target="_blank" title="TYPO3 Browser Forums. 500 TYPO3-integrators are registered.">
          TYPO3 Browser Forum &raquo;</a>
      </p>
      ';
		die( $prompt );
	}

	/**
	 * setction( )
	 *
	 * @param string
	 * @return void
	 * @access public
	 * @version 8.0.21
	 * @since 8.0.21
	 *
	 */
	public function setAction( $action )
	{
		$this->_sAction = $action;
	}

	/**
	 * setDRS( )
	 *
	 * @param Netzmacher\Browser\Utility\FrontendEditing\DRS $oDRS
	 * @return void
	 * @access public
	 * @version 8.0.17
	 * @since 8.0.17
	 *
	 */
	public function setDRS( \Netzmacher\Browser\Utility\FrontendEditing\DRS $oDRS )
	{
		$this->_oDRS = $oDRS;
	}

	/**
	 * setTCA( ) :
	 *
	 * @param Netzmacher\Browser\Utility\FrontendEditing\TCA $oTCA
	 * @return void
	 * @access public
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	public function setTCA( \Netzmacher\Browser\Utility\FrontendEditing\TCA $oTCA )
	{
		$this->_oTCA = $oTCA;
	}

	/**
	 * main( ) :
	 *
	 * @param array $settings current TypoScript
	 * @return void
	 * @access public
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	public function main( $settings )
	{
		$this->_aSettings = $settings;

		$this->_init();

		if( !$this->_requirements() )
		{
			return;
		}

		$this->_render();
	}

}
